import unittest
from pydrawio.mxgraphmodel import decompress, MxGraphModel

class TestMxGraphModel(unittest.TestCase):
    def test_decompress(self):
        tests = 'lJI9b8MgGIR/DWMlA1WaDl5i5WNIl2ZIOyLz1iDxJUJip7++ChCMoizdfA9nOO4F0U5PW8+c+LAcFCINn1pECMbNAhFy09ebXuJlkoOXPBpmeZC/EFGT2VlyOFWmYK0K0tWot8ZAHyrCvLdjbfmxqj7JsQEe5KFn6pEdJQ8iJiZvM92BHEQ6DS/eE9csGXPqk2DcjgXQNaKdtzakLz11oG71pNvPFkQ2ee2ZrwTzYMJ/foRud/naD+bTuOG4N99yp8lL3u3C1DleOwcP19SCt2fDgbcNoqtRyAAHx3poR88coisRtGoxoqvHSPddwQeYCsihtmA1BH9FpIlrZcjxVeDXrMZ76/hepyiNLzJhcbhD2bDuoRzzrBpENvMgkn1+sHT9BwAA//8BAAD//w=='
        mxgraphmodel = decompress(tests)
        self.assertEqual('0', mxgraphmodel.content.items[0].id)
        self.assertEqual('1', mxgraphmodel.content.items[1].id)
        self.assertEqual('eCHvXLgnRnpgWLnYiHm2-1', mxgraphmodel.content.items[2].id)

    def test_compress(self):
        tests = 'lJI9b8MgGIR/DWMlA1WaDl5i5WNIl2ZIOyLz1iDxJUJip7++ChCMoizdfA9nOO4F0U5PW8+c+LAcFCINn1pECMbNAhFy09ebXuJlkoOXPBpmeZC/EFGT2VlyOFWmYK0K0tWot8ZAHyrCvLdjbfmxqj7JsQEe5KFn6pEdJQ8iJiZvM92BHEQ6DS/eE9csGXPqk2DcjgXQNaKdtzakLz11oG71pNvPFkQ2ee2ZrwTzYMJ/foRud/naD+bTuOG4N99yp8lL3u3C1DleOwcP19SCt2fDgbcNoqtRyAAHx3poR88coisRtGoxoqvHSPddwQeYCsihtmA1BH9FpIlrZcjxVeDXrMZ76/hepyiNLzJhcbhD2bDuoRzzrBpENvMgkn1+sHT9BwAA//8BAAD//w=='
        mxgraphmodel = decompress(tests)
        mxgraphmodel = decompress(mxgraphmodel.compress())

    def test_get_ids(self):
        tests = 'lJI9b8MgGIR/DWMlA1WaDl5i5WNIl2ZIOyLz1iDxJUJip7++ChCMoizdfA9nOO4F0U5PW8+c+LAcFCINn1pECMbNAhFy09ebXuJlkoOXPBpmeZC/EFGT2VlyOFWmYK0K0tWot8ZAHyrCvLdjbfmxqj7JsQEe5KFn6pEdJQ8iJiZvM92BHEQ6DS/eE9csGXPqk2DcjgXQNaKdtzakLz11oG71pNvPFkQ2ee2ZrwTzYMJ/foRud/naD+bTuOG4N99yp8lL3u3C1DleOwcP19SCt2fDgbcNoqtRyAAHx3poR88coisRtGoxoqvHSPddwQeYCsihtmA1BH9FpIlrZcjxVeDXrMZ76/hepyiNLzJhcbhD2bDuoRzzrBpENvMgkn1+sHT9BwAA//8BAAD//w=='
        mxgraphmodel = decompress(tests)
        ids = mxgraphmodel.get_ids()
        self.assertEqual('0', ids[0])
        self.assertEqual('1', ids[1])
        self.assertEqual('eCHvXLgnRnpgWLnYiHm2-1', ids[2])

    def test_find_content(self):
        tests = 'lJI9b8MgGIR/DWMlA1WaDl5i5WNIl2ZIOyLz1iDxJUJip7++ChCMoizdfA9nOO4F0U5PW8+c+LAcFCINn1pECMbNAhFy09ebXuJlkoOXPBpmeZC/EFGT2VlyOFWmYK0K0tWot8ZAHyrCvLdjbfmxqj7JsQEe5KFn6pEdJQ8iJiZvM92BHEQ6DS/eE9csGXPqk2DcjgXQNaKdtzakLz11oG71pNvPFkQ2ee2ZrwTzYMJ/foRud/naD+bTuOG4N99yp8lL3u3C1DleOwcP19SCt2fDgbcNoqtRyAAHx3poR88coisRtGoxoqvHSPddwQeYCsihtmA1BH9FpIlrZcjxVeDXrMZ76/hepyiNLzJhcbhD2bDuoRzzrBpENvMgkn1+sHT9BwAA//8BAAD//w=='
        mxgraphmodel = decompress(tests)
        content = mxgraphmodel.find_content('eCHvXLgnRnpgWLnYiHm2-1')
        self.assertEqual('eCHvXLgnRnpgWLnYiHm2-1', content.id)

    def test_find_content_not_found(self):
        tests = 'lJI9b8MgGIR/DWMlA1WaDl5i5WNIl2ZIOyLz1iDxJUJip7++ChCMoizdfA9nOO4F0U5PW8+c+LAcFCINn1pECMbNAhFy09ebXuJlkoOXPBpmeZC/EFGT2VlyOFWmYK0K0tWot8ZAHyrCvLdjbfmxqj7JsQEe5KFn6pEdJQ8iJiZvM92BHEQ6DS/eE9csGXPqk2DcjgXQNaKdtzakLz11oG71pNvPFkQ2ee2ZrwTzYMJ/foRud/naD+bTuOG4N99yp8lL3u3C1DleOwcP19SCt2fDgbcNoqtRyAAHx3poR88coisRtGoxoqvHSPddwQeYCsihtmA1BH9FpIlrZcjxVeDXrMZ76/hepyiNLzJhcbhD2bDuoRzzrBpENvMgkn1+sHT9BwAA//8BAAD//w=='
        mxgraphmodel = decompress(tests)
        content = mxgraphmodel.find_content('aaaa')
        self.assertEqual(None, content)



import xml.etree.ElementTree as ET
from core import XmlObject
from typing import List

class Diagram(XmlObject):
    ''' Diagram
    Attributes:
        id (str)
        name (str)
        value (str)
    '''

    id: str
    name: str
    value: str

    def __init__(self, element: ET.Element):
        super().__init__(element)
        self.value = element.text

    def make_tree(self) -> ET.ElementTree:
        diagram = ET.Element('diagram')
        diagram = super().set_attrib(diagram)
        diagram.text = self.value

        return ET.ElementTree(diagram)

class Mxfile(XmlObject):
    ''' Mxfile
    Attributes:
        host (str)
        modified (str)
        agent (str)
        version (str)
        type (str)
        etag (str)
        diagram (List[Diagram])
    '''

    host: str
    modified: str
    agent: str
    version: str
    type: str
    etag: str
    diagram: List[Diagram]

    def __init__(self, xmlstr: str='<mxfile host="" modified="" agent="" version="" type="" etag=""></mxfile>'):
        tree = ET.ElementTree(ET.fromstring(xmlstr))
        root = tree.getroot()
        super().__init__(root)
        self.diagram = []
        diagrams = list(root)
        for diagram in diagrams:
            self.diagram.append(Diagram(diagram))

    def make_tree(self) -> ET.ElementTree:
        mxfile = ET.Element('mxfile')
        mxfile = super().set_attrib(mxfile)
        for diagram in self.diagram:
            mxfile.append(diagram.make_tree().getroot())

        return ET.ElementTree(mxfile)

    def write(self, filepath: str, encoding='utf-8'):
        ''' write
        Write Mxfile
        Args:
            filepath (str): Output filepath
            (optional) encoding (str): encode type
        '''

        self.make_tree().write(filepath, encoding=encoding)



import json
import xml.etree.ElementTree as ET
from typing import List
from core import XmlObject
from .mxgraphmodel import MxGraphModel

class Mxlibobject:
    ''' Mxlibobject
    Mxlibobject manage object of Mxlibrary value (list of json)
    Attributes:
        w (str): Width of chart
        h (str): Height of chart
        title (str): Title of chart
        xml (str): Compressed MxGraphModel xml string
    '''

    w: str
    h: str
    title: str
    xml: str

    def __init__(self, **entries):
        self.__dict__.update(entries)

class Mxlibrary(XmlObject):
    ''' Mxlibrary
    Mxlibrary manage Mxlibrary file
    Attributes:
        value (str): Mxlibrary value (json string)
        __mxlibobjects (List[Mxlibobject]): List of Mxlibobject
    '''

    value: str
    __mxlibobjects: List[Mxlibobject]

    def __init__(self, xmlstr: str ='<mxlibrary></mxlibrary>'):
        tree = ET.ElementTree(ET.fromstring(xmlstr))
        root = tree.getroot()
        self.value = root.text
        self.__mxlibobjects = []

    def make_tree(self) -> ET.ElementTree:
        mxlibrary = ET.Element('mxlibrary')
        mxlibrary.text = json.dumps(self.__mxlibobjects)

        return ET.ElementTree(mxlibrary)

    def set_mxlibobject(self, mxlibobjects: List[Mxlibobject]):
        ''' set_mxlibobject
        Set Mxlibobject to class variable of Mxlibrary
        Args:
            mxlibobjects (List[Mxlibobject]): List of Mxlibobject
        Examples:
            >>> mxlibrary.set_mxlibobject([ Mxlibobject({ value: '', 'w': 24, 'h': 24, 'title': 'test' }), ... ])
        '''

        self.__mxlibobjects = mxlibobjects

    def make_mxlibobject(self, mxgraphmodel: MxGraphModel, title: str, height: int, width: int) -> Mxlibobject:
        ''' make_mxlibobejct
        Make Mxlibobject from Mxgraphmodel and title, height, width
        Args:
            mxgraphmodel: Mxgraphmodel of chart
            title: title string
            height: Library object height
            width: Library object width
        Returns:
            Mxlibobject: Mxlibobject made from MxGraphmodel, title, height, width
        '''

        mxlibobj = {}
        mxlibobj['xml'] = mxgraphmodel.compress()
        mxlibobj['h'] = height
        mxlibobj['w'] = width
        mxlibobj['title'] = title

        return Mxlibobject(**mxlibobj)
    
    def write(self, filepath: str):
        ''' write
        Write Mxlibrary to specific filepath
        Args: filepath: output filepath string
        '''

        self.make_tree().write(filepath)



import base64
import zlib
import urllib.parse
import keyword
import xml.etree.ElementTree as ET
from typing import Dict, List
from abc import ABCMeta, abstractmethod
from core import XmlObject

class MxPoint(XmlObject):
    ''' MxPoint
    Attributes:
        x (str)
        y (str)
        as_ (str)
    '''

    x: str
    y: str
    as_: str

    def __init__(self, element: ET.Element):
        super().__init__(element)

    def make_tree(self) -> ET.ElementTree:
        mxpoint = ET.Element('mxPoint')

        return ET.ElementTree(self.set_attrib(mxpoint))

class MxGeometry(XmlObject):
    ''' MxGeometry
    Attributes:
        x (str)
        y (str)
        width (str)
        height (str)
        as_ (str)
        relative (str)
        mxPoint (MxPoint)
    '''

    x: str
    y: str
    width: str
    height: str
    as_: str
    relative: str
    mxPoint: MxPoint

    def __init__(self, element: ET.Element):
        super().__init__(element)
        mxpoint = element.find('mxPoint')
        self.mxPoint = MxPoint(mxpoint) if mxpoint else None

    def make_tree(self) -> ET.ElementTree:
        mxgeometry = ET.Element('mxGeometry')
        mxgeometry = self.set_attrib(mxgeometry)
        if self.mxPoint is not None:
            mxgeometry.append(self.mxPoint.make_tree().getroot())

        return ET.ElementTree(mxgeometry)

class IContent(XmlObject):
    ''' IContent
    Superclasses MxCell and Object
    Attributes:
        id (str)
    '''

    id: str
    def __init__(self, element: ET.Element):
        super().__init__(element)

    @abstractmethod
    def make_tree(self) -> ET.ElementTree:
        raise NotImplementedError

class MxCell(IContent):
    ''' MxCell
    Attributes:
        value (str)
        style (str)
        parent (str)
        source (str)
        target (str)
        edge (str)
        vertex (str)
        mxGeometry (MxGeometry)
    '''

    value: str
    style: str
    parent: str
    source: str
    target: str
    edge: str
    vertex: str
    mxGeometry: MxGeometry

    def __init__(self, element: ET.Element):
        super().__init__(element)
        mxgeometry = element.find('mxGeometry')
        self.mxGeometry = MxGeometry(mxgeometry) if mxgeometry else None

    def make_tree(self) -> ET.ElementTree:
        mxcell = ET.Element('mxCell')
        mxcell = self.set_attrib(mxcell)
        if self.mxGeometry is not None:
            mxcell.append(self.mxGeometry.make_tree().getroot())

        return ET.ElementTree(mxcell)

class Object(IContent):
    ''' Object
    Attributes:
        label (str)
        mxCell (MxCell)
    '''

    label: str
    mxCell: MxCell

    def __init__(self, element: ET.Element):
        super().__init__(element)
        object_ = element.find('object')
        self.mxCell = MxCell(element) if object_ else None

    def make_tree(self) -> ET.ElementTree:
        object_ = ET.Element('object')
        object_ = self.set_attrib(object_)
        if self.mxCell is not None:
            object_.append(self.mxCell.make_tree().getroot())

        return ET.ElementTree(object_)

class Root(XmlObject):
    ''' Root
    Attributes:
        items (List[IContent])
    '''

    items: List[IContent]

    def __init__(self, element: ET.Element):
        super().__init__(element)
        self.items = []
        contents = list(element)
        for content in contents:
            self.items.append(Object(content)) if content.find('mxCell') else self.items.append(MxCell(content))

    def make_tree(self) -> ET.ElementTree:
        root = ET.Element('root')
        for item in self.items:
            root.append(item.make_tree().getroot())

        return ET.ElementTree(root)

class MxGraphModel(XmlObject):
    ''' MxGraphModel
    Attributes:
        content (Root)
        dx (str)
        dy (str)
        grid (str)
        gridSize (str)
        guides (str)
        tooltips (str)
        connect (str)
        arrows (str)
        fold (str)
        page (str)
        pageScale (str)
        pageWidth (str)
        pageHeight (str)
        math (str)
        shadow (str)
    '''

    content: Root
    dx: str
    dy: str
    grid: str
    gridSize: str
    guides: str
    tooltips: str
    connect: str
    arrows: str
    fold: str
    page: str
    pageScale: str
    pageWidth: str
    pageHeight: str
    math: str
    shadow: str

    def __init__(self, xmlstr: str):
        tree = ET.ElementTree(ET.fromstring(xmlstr))
        root = tree.getroot()
        super().__init__(root)
        content = root.find('root')
        self.content = Root(content) if content else None

    def make_tree(self) -> ET.ElementTree:
        mxGraphModel = ET.Element('mxGraphModel')
        mxGraphModel = super().set_attrib(mxGraphModel)
        if self.content is not None:
            mxGraphModel.append(self.content.make_tree().getroot())

        return ET.ElementTree(mxGraphModel)

    def compress(self) -> str:
        ''' compress
        Compress MxGraphModel xml string
        Returns:
            str: Compressed string
        '''

        tree = self.make_tree()
        quoted = urllib.parse.quote(ET.tostring(tree.getroot()), safe='~()*!.\'')
        compress = zlib.compressobj(zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED, -15, memLevel=8, strategy=zlib.Z_DEFAULT_STRATEGY)
        compressed_b = compress.compress(quoted.encode())
        compressed_b += compress.flush()
        return base64.b64encode(compressed_b).decode()

    def get_ids(self) -> List[str]:
        ''' get_ids
        Get IContent ID list from MxGraphModel
        Returns:
            List[str]: IContent ID list
        '''

        return [ item.id for item in self.content.items ]

    def find_content(self, id_: str) -> IContent:
        ''' find_content
        Find IContent from MxGraphModel
        Args:
            id_ (str): Target IContent id
        Returns:
            IContent: First match IContent
        '''

        founds = list(filter(lambda item: item.id == id_, self.content.items))
        if len(founds) < 1:
            return None

        return founds[0]


def decompress(compressed_str: str) -> MxGraphModel:
    ''' decompress
    Decompress compressed MxGraphModel xml string
    Args:
        compressed_str (str): Compressed string
    Returns:
        MxGraphModel: Source MxGraphmOdel
    '''

    compressed_b = base64.b64decode(compressed_str.encode())
    compressed_b = zlib.decompress(compressed_b, wbits=-15)
    xmlstr = urllib.parse.unquote(compressed_b.decode())
    return MxGraphModel(xmlstr)



import pydrawio
from setuptools import setup, find_packages

DESCRIPTION = 'pydrawio: manage draw.io file'
NAME = 'pydrawio'
AUTHOR = 'Naoto Kageyama'
AUTHOR_EMAIL = 'rencker3@gmail.com'
URL = 'https://github.com/capybara-alt/pydrawio'
LICENSE = 'MIT'
DOWNLOAD_URL = 'https://github.com/capybara-alt/pydrawio'
VERSION = pydrawio.__version__
PYTHON_REQUIRES = '>=3.7'

PACKAGES = [
    'pydrawio'
]

CLASSIFIERS = [
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
]

with open('README.md', 'r') as fp:
    readme = fp.read()
long_description = readme

setup(
      name=NAME,
      author=AUTHOR,
      author_email=AUTHOR_EMAIL,
      maintainer=AUTHOR,
      maintainer_email=AUTHOR_EMAIL,
      description=DESCRIPTION,
      long_description=long_description,
      long_description_content_type='text/markdown',
      license=LICENSE,
      url=URL,
      version=VERSION,
      download_url=DOWNLOAD_URL,
      python_requires=PYTHON_REQUIRES,
      packages=PACKAGES,
      classifiers=CLASSIFIERS
)



import unittest
from pydrawio.mxfile import Mxfile
from pydrawio.mxgraphmodel import decompress

class TestMxfile(unittest.TestCase):
    def test_mxfile(self):
        tests = '<mxfile host="Electron" modified="2022-09-07T15:50:09.872Z" agent="5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) draw.io/20.2.8 Chrome/102.0.5005.167 Electron/19.0.15 Safari/537.36" etag="gzH0kVSIBYVuorkLmoRh" version="20.2.8" type="device"><diagram id="f0Od_lqmHJlGkMBThjxj" name="ページ1">jZJLT4QwEMc/DUcTCgbXq7i7HNaLHFaPDR1pkz5Ityzgp7fIlEeIiadOfzOdx38apbnqz5Y2/M0wkFESsz5KX6MkISTO/DGSYSIHcphAbQXDoAWU4hsQxkhbweC2CXTGSCeaLayM1lC5DaPWmm4b9mXktmpDa9iBsqJyT6+COY5TJE8LL0DUPFQm2fPkUTQE4yQ3TpnpVig9RmlujXGTpfoc5Che0GV6d/rDOzdmQbv/PIC8uH9cav2um/p60Z+iUMkDZrlT2eLA2KwbggLWtJrBmCSO0peOCwdlQ6vR2/mde8adkv5GvLlvKlQA66BfIWzyDEaBs4MPQe+8evwx5BHv3aI/CaLylfYZMoorr+fUiyreQGHCdVnAr2/1jdPjDw==</diagram></mxfile>'
        mxfile = Mxfile(tests)
        self.assertEqual('Electron', mxfile.host)
        self.assertEqual('2022-09-07T15:50:09.872Z', mxfile.modified)
        self.assertEqual('5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) draw.io/20.2.8 Chrome/102.0.5005.167 Electron/19.0.15 Safari/537.36', mxfile.agent)
        self.assertEqual('gzH0kVSIBYVuorkLmoRh', mxfile.etag)
        self.assertEqual('20.2.8', mxfile.version)
        self.assertEqual('device', mxfile.type)
        self.assertEqual('f0Od_lqmHJlGkMBThjxj', mxfile.diagram[0].id)

    def test_decompress_mxgraphmodel(self):
        tests = '<mxfile host="Electron" modified="2022-09-07T15:50:09.872Z" agent="5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) draw.io/20.2.8 Chrome/102.0.5005.167 Electron/19.0.15 Safari/537.36" etag="gzH0kVSIBYVuorkLmoRh" version="20.2.8" type="device"><diagram id="f0Od_lqmHJlGkMBThjxj" name="ページ1">jZJLT4QwEMc/DUcTCgbXq7i7HNaLHFaPDR1pkz5Ityzgp7fIlEeIiadOfzOdx38apbnqz5Y2/M0wkFESsz5KX6MkISTO/DGSYSIHcphAbQXDoAWU4hsQxkhbweC2CXTGSCeaLayM1lC5DaPWmm4b9mXktmpDa9iBsqJyT6+COY5TJE8LL0DUPFQm2fPkUTQE4yQ3TpnpVig9RmlujXGTpfoc5Che0GV6d/rDOzdmQbv/PIC8uH9cav2um/p60Z+iUMkDZrlT2eLA2KwbggLWtJrBmCSO0peOCwdlQ6vR2/mde8adkv5GvLlvKlQA66BfIWzyDEaBs4MPQe+8evwx5BHv3aI/CaLylfYZMoorr+fUiyreQGHCdVnAr2/1jdPjDw==</diagram></mxfile>'
        mxfile = Mxfile(tests)
        compressed = mxfile.diagram[0].value
        mxgraphmodel = decompress(compressed)
        self.assertEqual('0', mxgraphmodel.content.items[0].id)
        self.assertEqual('1', mxgraphmodel.content.items[1].id)

    def test_write(self):
        tests = '<mxfile host="Electron" modified="2022-09-07T15:50:09.872Z" agent="5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) draw.io/20.2.8 Chrome/102.0.5005.167 Electron/19.0.15 Safari/537.36" etag="gzH0kVSIBYVuorkLmoRh" version="20.2.8" type="device"><diagram id="f0Od_lqmHJlGkMBThjxj" name="ページ1">jZJLT4QwEMc/DUcTCgbXq7i7HNaLHFaPDR1pkz5Ityzgp7fIlEeIiadOfzOdx38apbnqz5Y2/M0wkFESsz5KX6MkISTO/DGSYSIHcphAbQXDoAWU4hsQxkhbweC2CXTGSCeaLayM1lC5DaPWmm4b9mXktmpDa9iBsqJyT6+COY5TJE8LL0DUPFQm2fPkUTQE4yQ3TpnpVig9RmlujXGTpfoc5Che0GV6d/rDOzdmQbv/PIC8uH9cav2um/p60Z+iUMkDZrlT2eLA2KwbggLWtJrBmCSO0peOCwdlQ6vR2/mde8adkv5GvLlvKlQA66BfIWzyDEaBs4MPQe+8evwx5BHv3aI/CaLylfYZMoorr+fUiyreQGHCdVnAr2/1jdPjDw==</diagram></mxfile>'
        mxfile = Mxfile(tests)
        mxfile.write('/dev/null')
